@@ -2,6 +2,7 @@
 
 from typing import Optional, TYPE_CHECKING
 
+from onapsdk.exceptions import ResourceNotFound
 from .instance import Instance
 
 if TYPE_CHECKING:
@@ -126,6 +127,8 @@ class PnfInstance(Instance):  # pylint: disable=too-many-instance-attributes
         self.pnf_ipv4_address: Optional[str] = pnf_ipv4_address
         self.pnf_ipv6_address: Optional[str] = pnf_ipv6_address
 
+        self._pnf: "Pnf" = None
+
     def __repr__(self) -> str:
         """Pnf instance object representation.
 
@@ -145,6 +148,30 @@ class PnfInstance(Instance):  # pylint: disable=too-many-instance-attributes
         """
         return f"{self.base_url}{self.api_version}/network/pnfs/pnf/{self.pnf_name}"
 
+    @property
+    def pnf(self) -> "Pnf":
+        """Pnf associated with that pnf instance.
+
+        Raises:
+            ResourceNotFound: Could not find PNF for that PNF instance
+
+        Returns:
+            Pnf: Pnf object associated with Pnf instance
+
+        """
+        if not self._pnf:
+            for pnf in self.service_instance.sdc_service.pnfs:
+                if pnf.model_version_id == self.model_version_id:
+                    self._pnf = pnf
+                    return self._pnf
+
+            msg = (
+                f'Could not find PNF for the PNF instance'
+                f' with model version ID "{self.model_version_id}"'
+            )
+            raise ResourceNotFound(msg)
+        return self._pnf
+
     @classmethod
     def create_from_api_response(cls, api_response: dict,
                                  service_instance: "ServiceInstance") -> "PnfInstance":
@@ -189,7 +216,7 @@ class PnfInstance(Instance):  # pylint: disable=too-many-instance-attributes
                    pnf_ipv4_address=api_response.get("pnf-ipv4-address"),
                    pnf_ipv6_address=api_response.get("pnf-ipv6-address"))
 
-    def delete(self) -> None:
+    def delete(self, a_la_carte: bool = True) -> None:
         """Delete Pnf instance.
 
         PNF deletion it's just A&AI resource deletion. That's difference between another instances.
