@@ -1,6 +1,7 @@
 """VF module instance."""
 
 from onapsdk.so.deletion import VfModuleDeletionRequest
+from onapsdk.exceptions import ResourceNotFound
 
 from .instance import Instance
 
@@ -62,10 +63,7 @@ class VfModuleInstance(Instance):  # pylint: disable=too-many-instance-attribute
             selflink (str, optional): Path to the controller object. Defaults to None.
         """
         super().__init__(resource_version=resource_version, model_version_id=model_version_id,
-                         model_invariant_id=model_invariant_id,
-                         persona_model_version=persona_model_version,
-                         widget_model_id=widget_model_id,
-                         widget_model_version=widget_model_version)
+                         model_invariant_id=model_invariant_id)
         self.vnf_instance: "VnfInstance" = vnf_instance
         self.vf_module_id: str = vf_module_id
         self.is_base_vf_module: bool = is_base_vf_module
@@ -77,6 +75,11 @@ class VfModuleInstance(Instance):  # pylint: disable=too-many-instance-attribute
         self.contrail_service_instance_fqdn: str = contrail_service_instance_fqdn
         self.module_index: int = module_index
         self.selflink: str = selflink
+        self.persona_model_version: str = persona_model_version
+        self.widget_model_id: str = widget_model_id
+        self.widget_model_version: str = widget_model_version
+
+        self._vf_module: "VfModule" = None
 
     def __repr__(self) -> str:
         """Object represetation.
@@ -103,14 +106,22 @@ class VfModuleInstance(Instance):  # pylint: disable=too-many-instance-attribute
     def vf_module(self) -> "VfModule":
         """Vf module associated with that vf module instance.
 
-        Raises:
-            AttributeError: Could not find VF module for that VfModule instance
-
         Returns:
             VfModule: VfModule object associated with vf module instance
 
         """
-        return self.vnf_instance.vnf.vf_module
+        if not self._vf_module:
+            for vf_module in self.vnf_instance.vnf.vf_modules:
+                if vf_module.model_version_id == self.model_version_id:
+                    self._vf_module = vf_module
+                    return self._vf_module
+
+            msg = (
+                f'Could not find VF modules for the VF Module instance'
+                f' with model version ID "{self.model_version_id}"'
+            )
+            raise ResourceNotFound(msg)
+        return self._vf_module
 
     @classmethod
     def create_from_api_response(cls,
@@ -146,14 +157,17 @@ class VfModuleInstance(Instance):  # pylint: disable=too-many-instance-attribute
             selflink=api_response.get("selflink")
         )
 
-    def delete(self) -> "VfModuleDeletionRequest":
+    def delete(self, a_la_carte: bool = True) -> "VfModuleDeletionRequest":
         """Create deletion request.
 
         Send request to delete VF module instance
 
+        Args:
+            a_la_carte (boolean): deletion mode
+
         Returns:
             VfModuleDeletionRequest: Deletion request object
 
         """
         self._logger.debug("Delete %s VF module", self.vf_module_id)
-        return VfModuleDeletionRequest.send_request(self)
+        return VfModuleDeletionRequest.send_request(self, a_la_carte)
